class Game(object):
    name = ''
    author = ''
    url = ''
    path = ''
    # actions that should be entered when the game is started to accept various prompts, warning and settings
    startup_actions = []

    def __init__(self):
        raise NotImplementedError("Game is an abstract class.")


class CustomGame(Game):
    def __init__(self, name, author='', url='', path='', startup_actions=None):
        if startup_actions is None:
            startup_actions = []
        self.name = name
        self.author = author
        self.url = url
        self.path = path
        self.startup_actions = startup_actions



import random

import time
from selenium.common.exceptions import StaleElementReferenceException, NoSuchElementException

from pyfiction.games.StarCourt.star_court import StarCourt
from pyfiction.simulators.html_simulator import HTMLSimulator
from pyfiction.simulators.simulator import UnknownEndingException


class StarCourtSimulator(HTMLSimulator):
    # the maximum number of steps the agent should take before we interrupt him to break infinite cycles
    max_steps = 500

    # the recommended number of random game walkthroughs for vocabulary initialization
    # should ideally cover all possible states and used words
    initialization_iterations = 4096

    # if the game rewards are in e.g. [-30, 30], set the reward scale to 30 so that the result is in [-1, 1]
    reward_scale = 30

    def __init__(self, shuffle_actions=True):
        super().__init__(StarCourt, shuffle_actions=shuffle_actions)
        self.actions = None

    def restart(self):
        super().restart()

    def write(self, action_index):
        action = self.actions[action_index][1]
        action.click()

    def read(self, tries=0, max_tries=10):

        try:
            # text is always in the the tw-story html tag:
            passage = self.driver.find_element_by_class_name("passage")
            text = passage.text

            # self.actions = [(action.text, action) for action in passage.find_elements_by_class_name("internalLink")]
            self.actions = [(action.text, action) for action in passage.find_elements_by_tag_name("a")]

            reward = -0.1

            # reward for using favors to upgrade the job/pet/house:
            if text.startswith('You get a job as a'):
                reward += 5

            # detect endings
            if len(self.actions) == 2 and self.actions[0][0] == "Take survey." and self.actions[1][0] == "Start over.":
                self.actions = []

            if not self.actions:
                ending = text

                # death but no sentence
                if ending.startswith(
                    'Here on the astral plane, your psychic bodies are as physical and real as your material body is in the physical world.'):
                    reward = -20
                # death sentence by trial (literally)
                elif ending.startswith(
                    'Nah. You die as poison consumes your body. And because you failed trial by poison, you\'re declared guilty!'):
                    reward = -30
                # happily ever after
                elif ending.startswith('You\'re all out of favors! I guess working as a'):
                    reward = 15
                # death sentence by trial (literally)
                elif ending.startswith(
                    'The only thing Pride finds more beautiful than itself is the destruction of those less beautiful than it!'):
                    reward = -30
                # death sentence by trial (literally)
                elif ending.startswith('Immediately upon starting the battle, the titanic creature falls asleep!'):
                    reward = -30
                # death sentence by trial (literally)
                elif ending.startswith('You are torn limb from limb by the many-limbed creature!'):
                    reward = -30
                # "blasting you right in the core"
                elif ending.startswith('You remember you training at Psi City and concentrate'):
                    reward = -30
                # innocent
                elif ending.startswith('And so you do, spacer, so you do.'):
                    reward = 15
                # speaks for itself
                elif ending.startswith("BLAMMO!!\n\nYou're dead! And what's worse, you're guilty!"):
                    reward = -30

                # sentenced but alive, determine the reward based on length of the sentence: (0 < length < 2000) years mapped to (-20 < reward < 0)
                elif 'The Judge bangs their laser gavel a final time. "Robailiff, you may take the prisoner away."' in ending:
                    words = ending.split()
                    years = int(words[words.index('years') - 1])
                    reward = -years / 100

                # one of the many endings of guilt and death
                elif "You're dead! I guess that means you're guilty!" in ending:
                    reward = -30
                # dead but not guilty
                elif 'You are neither guilty nor innocent, as law has been dethroned in the universe. However, you have died in a fire.' in ending:
                    reward = -20
                # escaped
                elif "You let Star Court evaporate like a bad memory. You're on the other side of the sector by the time they notice you're missing." in ending:
                    reward = 10
                # burned
                elif 'How does Star Court generate this much trash, you think as you burn.' in ending:
                    reward = -20
                # death
                elif 'You got smoked by a crime ghost.' in ending:
                    reward = -20
                # death
                elif "Congratulations, you're innocent! You're also dead." in ending:
                    reward = -20
                # dead and guilty
                elif "The knife hits you right between the eyes. You are killed immediately, and, what's worse, you're found guilty by the court!" in ending:
                    reward = -30
                # dead and guilty
                elif "means you're guilty!" in ending:
                    reward = -30

                else:
                    raise UnknownEndingException('Unknown ending text, cannot assign reward: ', ending)

            elif self.shuffle_actions:
                random.shuffle(self.actions)

        except (UnknownEndingException, NoSuchElementException, StaleElementReferenceException) as e:

            if tries == 0:
                print('WARNING, simulator exception:', e)

            if tries < max_tries:
                print('Trying to read again after a short wait, try', tries + 1, 'out of', max_tries)
                time.sleep(0.1)
                return self.read(tries=tries + 1)
            else:
                raise e

        return text, [action[0] for action in self.actions], reward

    def close(self):
        self.driver.close()


if __name__ == '__main__':
    simulator = StarCourtSimulator()
    starts = 0
    ends = 0

    for i in range(100):

        total_reward = 0

        while True:

            state, actions, reward = simulator.read()
            total_reward += reward

            if state.startswith('‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌‌The laser gavel pounds.'):
                starts += 1

            # print(state)
            # print('actions: ', actions)
            # print(reward)
            # print('-----------------------------')

            if not actions:
                print(state)
                print('last', reward, 'total', total_reward)
                print('--------------------')
                ends += 1
                break

            action = random.randint(0, len(actions) - 1)
            simulator.write(action)

        simulator.restart()

    simulator.driver.close()
    print('starts', starts, 'ends', ends)



import re

from pyfiction.games.MachineOfDeath.machine_of_death import MachineOfDeath
from pyfiction.simulators.simulator import Simulator
from pyfiction.simulators.text_games.simulators.MySimulator import MachineOfDeathSimulator as MODS


class MachineOfDeathSimulator(Simulator):
    """
    Simulator wrapper for the game Machine of Death.
    When importing this class, it is necessary to import StoryNode (!):
    from pyfiction.simulators.text_games.simulators.MySimulator import StoryNode
    """

    # the maximum number of steps the agent should take before we interrupt him to break infinite cycles
    max_steps = 500

    # the recommended number of random game walkthroughs for vocabulary initialization
    # should ideally cover all possible states and used words
    initialization_iterations = 8192

    # if the game rewards are in e.g. [-30, 30], set the reward scale to 30 so that the result is in [-1, 1]
    reward_scale = 30

    def __startup_actions(self):
        pass

    def restart(self):
        self.simulator.Restart()

    def __init__(self, shuffle_actions=True, paraphrase_actions=False):
        self.game = MachineOfDeath
        self.simulator = MODS(shuffle_actions, doParaphrase=paraphrase_actions)

    def read(self, **kwargs):
        state, actions, reward = self.simulator.Read()
        # the original simulator does not remove some HTML tags - remove them all:
        state = re.sub('<[^>]*>', '', state)
        return state, actions, reward

    def write(self, index):
        """
        Accepts an INDEX of an action, not its contents (i.e. not a string or a text description).
        """
        self.simulator.Act(index)



# source: http://eyalarubas.com/python-subproc-nonblock.html

from queue import Queue, Empty
from threading import Thread


class NonBlockingStreamReader:
    def __init__(self, stream):
        '''
        stream: the stream to read from.
                Usually a process' stdout or stderr.
        '''

        self._s = stream
        self._q = Queue()

        def _populate_queue(stream, queue):
            '''
            Collect lines from 'stream' and put them in 'queue'.
            '''

            while True:
                line = stream.readline().decode('utf-8', 'replace')
                if line:
                    queue.put(line)
                else:
                    raise UnexpectedEndOfStream

        self._t = Thread(target=_populate_queue,
                         args=(self._s, self._q))
        self._t.daemon = True
        self._t.start()  # start collecting lines from the stream

    def read_line(self, timeout=None):
        try:
            return self._q.get(block=timeout is not None,
                               timeout=timeout)
        except Empty:
            return None


class UnexpectedEndOfStream(Exception): pass



import os

from pyfiction.interpreters.interpreter import Interpreter


class Glulxe(Interpreter):
    name = 'glulxe'
    description = 'cheapglulxe - cheapglk + glulxe compiled interpreter.'
    path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cheapglulxe')



from subprocess import *

from pyfiction.interpreters.glulx.glulxe import Glulxe
from pyfiction.simulators.nbstreamreader import NonBlockingStreamReader
from pyfiction.simulators.simulator import Simulator


class GlulxSimulator(Simulator):
    interpreter = Glulxe

    def __init__(self, game):
        self.game = game
        self.stream = None
        self.stream_reader = None

    def restart(self):

        print('Running interpreter ', self.interpreter.path, ' on game ', self.game.path)
        self.stream = Popen([self.interpreter.path, self.game.path], stdin=PIPE, stdout=PIPE, stderr=STDOUT,
                            bufsize=0, universal_newlines=False)

        self.stream_reader = NonBlockingStreamReader(self.stream.stdout)

        self.__startup_actions()
        print('Game started')

    def __startup_actions(self):
        for action in self.game.__startup_actions:
            # print(self.read())
            self.read()
            self.write(action)
            # print('--------------------------------------')
            # print('all startup actions have been executed')
            # print('--------------------------------------')

    def write(self, text):
        # print('writing->', repr(text), '<-')
        self.stream.stdin.flush()
        self.stream.stdin.write(text.encode('utf-8'))
        # print('write end')

    # TODO: add a regexp 'timeout' (e.g. read until '\n >' is read)
    def read(self, timeout=0.01):
        # print('read start')
        lines = []
        while True:
            line = self.stream_reader.read_line(timeout)
            if line is not None:
                if not (line == '\n'):
                    lines.append(line)
                # print(output)
                self.stream.stdout.flush()
            else:
                # print('[No more data]')
                break
                # print(output)

        # print('read end')
        return lines



import random

import time

from selenium.common.exceptions import NoSuchElementException

from pyfiction.games.CatSimulator2016.cat_simulator_2016 import CatSimulator2016
from pyfiction.simulators.html_simulator import HTMLSimulator
from pyfiction.simulators.simulator import UnknownEndingException


class CatSimulator2016Simulator(HTMLSimulator):
    # the maximum number of steps the agent should take before we interrupt him to break infinite cycles
    max_steps = 100

    # the recommended number of random game walkthroughs for vocabulary initialization
    # should ideally cover all possible states and used words
    initialization_iterations = 256

    # if the game rewards are in e.g. [-30, 30], set the reward scale to 30 so that the result is in [-1, 1]
    reward_scale = 20

    def __init__(self, shuffle_actions=True):
        super().__init__(CatSimulator2016, shuffle_actions=shuffle_actions)
        self.actions = None

    def restart(self):
        super().restart()

    def write(self, action_index):
        action = self.actions[action_index][1]
        action.click()

    def read(self, tries=0, max_tries=10):

        try:

            # text is always in the the tw-story html tag:
            text = self.driver.find_element_by_tag_name("tw-story").text

            self.actions = []

            try:
                # actions are always of one of the two class sets below:
                self.actions += [(action.text, action) for action in
                                 self.driver.find_elements_by_tag_name("tw-link")]

                self.actions += [(action.text, action) for action in
                                 self.driver.find_elements_by_css_selector(
                                     "tw-enchantment[class='link enchantment-link']")]
            except:
                pass

            reward = -0.1

            # detect good endings (bad endings have no actions)
            if len(self.actions) == 1 and self.actions[0][0] == "spoilers":
                self.actions = []

            if not self.actions:
                ending = text.lower()

                # remove the first line of the ending (only contains a back arrow):
                ending = ending[(ending.index('\n') + 1):]

                # slept on the stool but no food
                if ending.startswith('this was a good idea'):
                    reward = 0
                # fell down and slept briefly - no food
                elif ending.startswith('as good a place as any'):
                    reward = -20
                # killed the bird
                elif ending.startswith('mine!'):
                    reward = 10
                # fell into the sink - no food and no sleep
                elif ending.startswith('catlike reflexes'):
                    reward = -20
                # failed to hunt the bird - no food and no sleep
                elif ending.startswith('finish this'):
                    reward = -20
                # befriended the bird - food and sleep
                elif ending.startswith('friendship'):
                    reward = 20
                # no food, slept on the counter
                elif ending.startswith('not this time, water'):
                    reward = 10
                # slept outside, no food
                elif ending.startswith('serendipity'):
                    reward = 10
                else:
                    raise UnknownEndingException('Unknown ending text, cannot assign reward: ', ending)

            elif self.shuffle_actions:
                random.shuffle(self.actions)

        except (UnknownEndingException, NoSuchElementException) as e:

            if tries == 0:
                print('WARNING, simulator exception:', e)

            if tries < max_tries:
                print('Trying to read again after a short wait, try', tries + 1, 'out of', max_tries)
                time.sleep(0.1)
                return self.read(tries=tries + 1)
            else:
                raise e

        return text, [action[0] for action in self.actions], reward

    def close(self):
        self.driver.close()


if __name__ == '__main__':
    simulator = CatSimulator2016Simulator()

    for i in range(16):

        while True:

            try:
                state, actions, reward = simulator.read()
            except:
                pass

            print(state)
            for action in actions:
                print(action)
            print('actions: ', actions)
            print(reward)
            print('-----------------------------')

            if not actions:
                break

            action = random.randint(0, len(actions) - 1)
            simulator.write(action)

        simulator.restart()

    simulator.driver.close()



import random

from pyfiction.games.HowlingDogs.howling_dogs import HowlingDogs
from pyfiction.simulators.html_simulator import HTMLSimulator


class HowlingDogsSimulator(HTMLSimulator):

    # the maximum number of steps the agent should take before we interrupt him to break infinite cycles
    max_steps = 1000

    # the recommended number of random game walkthroughs for vocabulary initialization
    # should ideally cover all possible states and used words
    initialization_iterations = 1024

    # if the game rewards are in e.g. [-30, 30], set the reward scale to 30 so that the result is in [-1, 1]
    reward_scale = 10

    def __init__(self, shuffle_actions=True):
        super().__init__(HowlingDogs, shuffle_actions=shuffle_actions)
        self.restart()

    def restart(self):
        self.driver.execute_script('state.restart()')
        # self.driver.get('file:///' + self.game.path)
        self.startup_actions()

    def write(self, action_index):
        action = self.actions[action_index]
        self.driver.find_element_by_link_text(action).click()

    def read(self):

        # text is always here:
        text = self.driver.find_element_by_css_selector("div[id='passages']").text

        # actions are always of one of the two class sets below:
        self.actions = [action.text for action in self.driver.find_elements_by_class_name("internalLink")]

        try:
            back = self.driver.find_element_by_class_name("back")
        except:
            back = None
        if back:
            self.actions.append(back.text)

        reward = -0.1

        if not self.actions:
            ending = text.lower()
            if ending.startswith('for everyone who feels that way'):
                reward = 10
            elif ending.startswith('howling dogs by porpentine'):
                reward = -10
            else:
                print('Game ended and no actions left but an unknown ending reached, cannot assign reward: ',
                      ending)

        elif self.shuffle_actions:
            random.shuffle(self.actions)

        return text, self.actions, reward

    def startup_actions(self):
        super(HowlingDogsSimulator, self).startup_actions()

    def close(self):
        self.driver.close()


if __name__ == '__main__':
    simulator = HowlingDogsSimulator()

    for i in range(16):

        while True:
            state, actions, reward = simulator.read()

            print(state)
            for action in actions:
                print(action)
            print(reward)

            if not actions:
                break

            action = random.randint(0, len(actions) - 1)
            simulator.write(action)
            print(actions[action])
            print('-------------------')

        print('****************************************************************')

        simulator.restart()

    simulator.driver.close()



class Agent(object):
    def __init__(self):
        raise NotImplementedError("Agent is an abstract class.")

    def act(self, **kwargs):
        raise NotImplementedError("Agent is an abstract class.")



from pyfiction.simulators.simulator import Simulator


class ZMachineSimulator(Simulator):
    def __init__(self):
        raise NotImplementedError

    def restart(self, interpreter_path=None, game_path=None):
        raise NotImplementedError

    def __startup_actions(self):
        raise NotImplementedError

    def read(self, timeout=0.01):
        raise NotImplementedError

    def write(self, text):
        raise NotImplementedError



class Simulator:

    # the maximum number of steps the agent should take before we interrupt him to break infinite cycles
    max_steps = 500

    # the recommended number of random game walkthroughs for vocabulary initialization
    # should ideally cover all possible states and used words
    initialization_iterations = 1024

    # if the game rewards are in e.g. [-30, 30], set the reward scale to 30 so that the result is in [-1, 1]
    reward_scale = 1

    # reference to the underlying game class
    game = None

    def __init__(self):
        raise NotImplementedError("Game is an abstract class.")

    def restart(self):
        raise NotImplementedError("Game is an abstract class.")

    def startup_actions(self):
        raise NotImplementedError("Game is an abstract class.")

    def write(self, text):
        raise NotImplementedError("Game is an abstract class.")

    def read(self, timeout=0.01):
        raise NotImplementedError("Game is an abstract class.")


class UnknownEndingException(Exception):
    """Reached a state without any actions but cannot assign a reward, unknown state"""



import os

from pyfiction.games.game import Game


class Six(Game):

    def __init__(self):
        pass

    name = 'Six'
    author = 'Wade Clarke'
    path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'Six.gblorb')

    # issue extra SPACE commands just to be sure
    startup_actions = [' \n', ' \n', ' \n', ' \n', '1\n', ' \n']



import random

import time
from selenium.common.exceptions import NoSuchElementException

from pyfiction.games.TheRedHair.the_red_hair import TheRedHair
from pyfiction.simulators.html_simulator import HTMLSimulator
from pyfiction.simulators.simulator import UnknownEndingException


class TheRedHairSimulator(HTMLSimulator):
    # the maximum number of steps the agent should take before we interrupt him to break infinite cycles
    max_steps = 100

    # the recommended number of random game walkthroughs for vocabulary initialization
    # should ideally cover all possible states and used words
    initialization_iterations = 64

    # if the game rewards are in e.g. [-30, 30], set the reward scale to 30 so that the result is in [-1, 1]
    reward_scale = 20

    def __init__(self, shuffle_actions=True):
        super(TheRedHairSimulator, self).__init__(TheRedHair, shuffle_actions=shuffle_actions)

    def restart(self):
        # super().restart()
        # must restart in loop in case the button is not available yet
        while True:
            restarted = True
            try:
                self.driver.find_element_by_id("restart").click()
            except:
                restarted = False
            if restarted:
                break

    def write(self, action_index):
        action = self.actions[action_index][1]
        # action.click()
        # click in javascript instead (in chrome, clicks don't work here for some strange reason - hidden by ps or divs)
        self.driver.execute_script("arguments[0].click();", action)

    def read(self, tries=0, max_tries=10):

        try:
            # text is always in the last div:
            text = (self.driver.find_elements_by_css_selector("div")[-1]).text

            self.actions = []

            # don't fail if there are no actions of the second class
            try:
                # actions are always of one of the two class sets below:
                self.actions += [(action.text, action) for action in
                                 self.driver.find_elements_by_css_selector("a[class='squiffy-link link-section']")]
                self.actions += [(action.text, action) for action in
                                 self.driver.find_elements_by_css_selector("a[class='squiffy-link link-passage']")]
            except:
                pass

            reward = -0.1

            if not self.actions:
                ending = text.lower()
                if ending.startswith('you lose'):
                    reward = -10
                elif ending.endswith('all there is left is a red hair') or ending.endswith(
                    'it was the clown statue missing'):
                    reward = -20
                elif text.lower().startswith(
                    'you stay in the bedroom and eventually the parents come back and thank you'):
                    reward = 20
                else:
                    raise UnknownEndingException('Unknown ending text, cannot assign reward: ', ending)

            elif self.shuffle_actions:
                random.shuffle(self.actions)

        except (UnknownEndingException, NoSuchElementException) as e:

            if tries == 0:
                print('WARNING, simulator exception:', e)

            if tries < max_tries:
                print('Trying to read again after a short wait, try', tries + 1, 'out of', max_tries)
                time.sleep(0.1)
                return self.read(tries=tries + 1)
            else:
                raise e

        return text, [action[0] for action in self.actions], reward

    def close(self):
        self.driver.close()


if __name__ == '__main__':
    simulator = TheRedHairSimulator()

    for i in range(16):

        while True:
            state, actions, reward = simulator.read()

            print(state)
            for action in actions:
                print(action)
            print(reward)

            if not actions:
                break

            action = random.randint(0, len(actions) - 1)
            simulator.write(action)

        simulator.restart()

    simulator.driver.close()



import random
import time

# StoryNode import is needed for unpickling the game file:
import numpy as np
from multiprocessing import Process
from pyfiction.agents import agent
from pyfiction.simulators.games.catsimulator2016_simulator import CatSimulator2016Simulator
from pyfiction.simulators.games.howlingdogs_simulator import HowlingDogsSimulator
from pyfiction.simulators.games.machineofdeath_simulator import MachineOfDeathSimulator
from pyfiction.simulators.games.savingjohn_simulator import SavingJohnSimulator
from pyfiction.simulators.games.starcourt_simulator import StarCourtSimulator
from pyfiction.simulators.games.theredhair_simulator import TheRedHairSimulator
from pyfiction.simulators.games.transit_simulator import TransitSimulator

from pyfiction.simulators.text_games.simulators.MySimulator import StoryNode


class RandomSearchAgent(agent.Agent):
    """
    This agent randomly searches the action space and remembers:
     - the best trace and the best final cumulative reward for each state
         - best trace is a list of all game states and actions resulting in the largest cumulative reward
     - list of all endings
     - average reward
     - other useful statistic...

    This agent is very useful for determining various parameters of unknown games.
    """

    def __init__(self):
        random.seed(0)

        self.bestReward = -np.math.inf
        self.bestTrace = []

        # reward and trace for one episode
        self.totalReward = 0
        self.trace = []

        self.endings = []
        # experience is a set of all traces, where a trace is a set of tuples: [state, action, reward]
        self.experience = []

        self.states = []
        self.rewards = []

        self.reset()

    def act(self, state, actions, reward):

        self.totalReward += reward

        action = None
        index = None

        if actions:
            index = random.randint(0, len(actions) - 1)
            action = actions[index]
            # print('choosing action:', action)

        self.trace.append([state, actions, reward, action])
        return index

    def reset(self):

        if not self.trace:
            return

        if self.totalReward > self.bestReward:
            self.bestReward = self.totalReward
            self.bestTrace = self.trace
            print('new best reward : {0:10.3f}'.format(self.bestReward))
            print('new best actions: ', [x[3] for x in self.bestTrace])
            print('last state: ', self.trace[-1][0])
            print('----')
            print(self.trace[-10:])
            print('----')

        for state, _, _, _ in self.trace:
            if state in self.states:
                index = self.states.index(state)
                self.rewards[index] = max(self.totalReward, self.rewards[index])
            else:
                self.states.append(state)
                self.rewards.append(self.totalReward)

                # self.experience.append(self.trace)

        ending = self.trace[-1][0]

        if ending not in self.endings:
            # print('new ending: ', ending)
            self.endings.append(ending)
            # print('endings count: ', len(self.endings))

        # print()
        # print()

        self.totalReward = 0
        self.trace = []


def run(simulator, file_name, episodes, runs_per_episode, max_steps):
    agent = RandomSearchAgent()
    start_time = time.time()

    rewards = []
    words = 0
    descriptions = 0
    positives = 0

    for i in range(episodes):

        episode_rewards = []
        print('episode', i)

        for _ in range(runs_per_episode):

            steps = 0

            while True and steps < max_steps:

                (text, actions, reward) = simulator.read()
                # print(text, actions, reward)
                words += len(text.split())
                descriptions += 1

                player_input = agent.act(text, actions, reward)

                if player_input is None:
                    if agent.totalReward == 0:
                        agent.reset()
                        simulator.restart()
                        steps = 0
                        continue
                    else:
                        if agent.totalReward > 0:
                            positives += 1
                        episode_rewards.append(agent.totalReward)
                        agent.reset()
                        simulator.restart()
                        break

                else:
                    simulator.write(player_input)

                steps += 1

        rewards.append(episode_rewards)

    print('positive rewards', positives)

    with open(file_name, 'w') as f:
        # save rewards:
        # for episode_rewards in rewards:
        #     for reward in episode_rewards:
        #         f.write('{:.1f}'.format(reward) + ' ')
        #     f.write(',\n')
        f.write('{:.2f}'.format(words / descriptions))

    end_time = time.time()
    print("Duration: " + str(end_time - start_time))
    #
    # print()
    # print('Best rewards for all states:')
    # print(len(agent.states), ' states, ', len(agent.rewards), ' rewards')
    # print(agent.rewards)
    # print(list(zip(agent.rewards, agent.states)))
    # print()
    #
    # endings = agent.endings
    #
    # print('ENDINGS:', len(endings), ' ----------------------------- ')
    # for ending in endings:
    #     print(ending)
    #     print('*********************')


def runInParallel(*fns):
    proc = []
    for fn in fns:
        p = Process(target=fn)
        p.start()
        proc.append(p)
    for p in proc:
        p.join()


def main():
    MAsimulator = MachineOfDeathSimulator()
    # SAsimulator = SavingJohnSimulator()
    # THsimulator = TheRedHairSimulator()
    # simulator = HowlingDogsSimulator()
    # CAsimulator = CatSimulator2016Simulator()
    # TRsimulator = TransitSimulator()
    # STsimulator = StarCourtSimulator()

    episodes = 10000
    runs_per_episode = 1
    # max_steps = 500

    runInParallel(
        run(MAsimulator, '2random.txt', episodes, runs_per_episode, 500),
        # run(SAsimulator, '3random.txt', episodes, runs_per_episode, 100),
        # run(THsimulator, '5random.txt', episodes, runs_per_episode, 100),
        # run(CAsimulator, '1random.txt', episodes, runs_per_episode, 100),
        # run(TRsimulator, '6random.txt', episodes, runs_per_episode, 100),
        # run(STsimulator, '4random.txt', episodes, runs_per_episode, 500),
    )


if __name__ == "__main__":
    main()



from selenium import webdriver

from pyfiction.simulators.simulator import Simulator


class HTMLSimulator(Simulator):
    def __init__(self, game, shuffle_actions=True):
        self.game = game
        self.shuffle_actions = shuffle_actions

        # use a headless Chromium/Chrome browser
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("headless")
        chrome_options.add_argument("disable-gpu")
        self.driver = webdriver.Chrome(chrome_options=chrome_options)

        self.driver.get('file:///' + self.game.path)

    def restart(self):
        self.driver.get('file:///' + self.game.path)

    def startup_actions(self):
        for action in self.game.startup_actions:
            self.read()
            self.write(action)

    def write(self, text):
        pass

    def read(self):
        pass
